// Tiny string-based Symbol polyfill
  /* istanbul ignore next */
  if (typeof Symbol !== 'function') {
      var _symbol = function _Symbol() {
          return Math.random().toString(16).slice(2);
      };
      _symbol.for = function (key) {
          return key;
      };
      _symbol.iterator = '@@iterator';
      var global$1 = Function('return this')();
      Object.defineProperty(global$1, 'Symbol', { value: _symbol });
  }

// The default growth policy grows the buffer by 1.5x the requested capacity.
  var defaultGrowthPolicy = function defaultGrowthPolicy(minCapacity) {
      return 3 * minCapacity >>> 1;
  };
  // The default allocator simply creates a new array buffer.
  var defaultAllocator = function defaultAllocator(byteLength) {
      return new ArrayBuffer(byteLength);
  };
  // The default dealloctor does nothing.
  var defaultDeallocator = function defaultDeallocator(buffer) {};
  // The default reallocator uses the ArrayBuffer::transfer function, if it is available.
  // Otherwise it simply allocates a new buffer and copies the old data over.
  var defaultReallocator = ArrayBuffer['transfer'] || function defaultReallocator(oldBuffer, newByteLength, allocator, deallocator) {
      var newBuffer = allocator(newByteLength);
      // Copy data over to the new buffer
      var oldView = new Uint8Array(oldBuffer);
      var newView = new Uint8Array(newBuffer);
      if (newView.length <= oldView.length) {
          oldView = oldView.subarray(0, newView.length);
      }
      newView.set(oldView);
      deallocator(oldBuffer);
      return newBuffer;
  };

var classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  var createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

var ArrayIterator = function () {
      function ArrayIterator(array, start, end) {
          classCallCheck(this, ArrayIterator);

          this._array = array;
          this._start = start;
          this._end = end;
          this._result = { value: array[start], done: false };
      }

      createClass(ArrayIterator, [{
          key: 'next',
          value: function next() {
              if (this._start < this._end) {
                  this._result.value = this._array[this._start++];
                  this._result.done = false;
              } else {
                  this._result.value = undefined;
                  this._result.done = true;
              }
              return this._result;
          }
      }]);
      return ArrayIterator;
  }();
  function isTypedArrayConstructor(obj) {
      switch (obj) {
          case Int8Array:
          case Uint8Array:
          case Uint8ClampedArray:
          case Int16Array:
          case Uint16Array:
          case Int32Array:
          case Uint32Array:
          case Float32Array:
          case Float64Array:
              return true;
      }
      return false;
  }
  function isNumber(obj) {
      return typeof obj === 'number';
  }
  function isFunction(obj) {
      return typeof obj === 'function';
  }
  function isArrayLike(obj) {
      return typeof obj !== 'function' && typeof obj.length === 'number';
  }
  function isIterable(obj) {
      return Symbol.iterator in obj && typeof obj[Symbol.iterator] === 'function';
  }

var DynamicTypedArray = function () {
      function DynamicTypedArray() {
          var constructor = arguments.length <= 0 || arguments[0] === undefined ? Float64Array : arguments[0];
          var init = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

          var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

          var _ref$growthPolicy = _ref.growthPolicy;
          var growthPolicy = _ref$growthPolicy === undefined ? defaultGrowthPolicy : _ref$growthPolicy;
          var _ref$allocator = _ref.allocator;
          var allocator = _ref$allocator === undefined ? defaultAllocator : _ref$allocator;
          var _ref$deallocator = _ref.deallocator;
          var deallocator = _ref$deallocator === undefined ? defaultDeallocator : _ref$deallocator;
          var _ref$reallocator = _ref.reallocator;
          var reallocator = _ref$reallocator === undefined ? defaultReallocator : _ref$reallocator;
          classCallCheck(this, DynamicTypedArray);

          if (!isTypedArrayConstructor(constructor)) {
              throw new TypeError('The constructor must be a typed array constructor function!');
          }
          if (!isFunction(growthPolicy)) {
              throw new TypeError('The growthPolicy must be a function!');
          }
          if (!isFunction(allocator)) {
              throw new TypeError('The allocator must be a function!');
          }
          if (!isFunction(deallocator)) {
              throw new TypeError('The deallocator must be a function!');
          }
          if (!isFunction(reallocator)) {
              throw new TypeError('The reallocator must be a function!');
          }
          this._array = undefined;
          this._length = 0;
          this._constructor = constructor;
          this._growthPolicy = growthPolicy;
          this._allocator = allocator;
          this._deallocator = deallocator;
          this._reallocator = reallocator;
          if (isNumber(init)) {
              if (init < 0) {
                  throw new RangeError('The initial size must not be negative!');
              }
              this._ensureCapacity(init);
              this._length = init;
          } else if (isArrayLike(init)) {
              var length = init.length;

              this._ensureCapacity(length);
              this._length = length;
              for (var i = 0; i < length; ++i) {
                  this._array[i] = init[i];
              }
          } else if (isIterable(init)) {
              var _iteratorNormalCompletion = true;
              var _didIteratorError = false;
              var _iteratorError = undefined;

              try {
                  for (var _iterator = init[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                      var value = _step.value;

                      this.push(value);
                  }
              } catch (err) {
                  _didIteratorError = true;
                  _iteratorError = err;
              } finally {
                  try {
                      if (!_iteratorNormalCompletion && _iterator.return) {
                          _iterator.return();
                      }
                  } finally {
                      if (_didIteratorError) {
                          throw _iteratorError;
                      }
                  }
              }
          } else {
              throw new TypeError('The initilizer must be a number or an iterable object.');
          }
      }

      createClass(DynamicTypedArray, [{
          key: 'size',
          value: function size() {
              return this._length;
          }
      }, {
          key: 'capacity',
          value: function capacity() {
              return this._array.length;
          }
      }, {
          key: 'get',
          value: function get(index) {
              if (index < this._length) {
                  return this._array[index];
              }
          }
      }, {
          key: 'set',
          value: function set(index, value) {
              if (index < this._length) {
                  this._array[index] = value;
              }
          }
      }, {
          key: 'push',
          value: function push() {
              var length = arguments.length;

              this._ensureCapacity(this._length + length);
              for (var i = 0; i < length; ++i) {
                  this._array[this._length++] = arguments[i];
              }
          }
      }, {
          key: 'pop',
          value: function pop() {
              return this._length <= 0 ? undefined : this._array[--this._length];
          }
      }, {
          key: 'forEach',
          value: function forEach(callback, thisArg) {
              var length = this._length;
              for (var i = 0; i < length; ++i) {
                  callback.call(thisArg, this._array[i], i);
              }
          }
      }, {
          key: Symbol.iterator,
          value: function value() {
              return new ArrayIterator(this._array, 0, this._length);
          }
      }, {
          key: 'toArray',
          value: function toArray() {
              return Array.prototype.slice.call(this._array, 0, this._length);
          }
      }, {
          key: 'toJSON',
          value: function toJSON() {
              return this.toArray();
          }
      }, {
          key: 'toString',
          value: function toString() {
              return this.toArray().toString();
          }
      }, {
          key: '_ensureCapacity',
          value: function _ensureCapacity(minCapacity) {
              if (!this._array || minCapacity > this.capacity()) {
                  var newCapacity = this._growthPolicy(minCapacity);
                  if ((newCapacity | 0) !== newCapacity) {
                      throw new TypeError('The computed new capacity (' + newCapacity + ') is not an integer value.');
                  } else if (newCapacity < minCapacity) {
                      throw new RangeError('The computed new capacity (' + newCapacity + ') is smaller than the requested minimum capacity (' + minCapacity + ')');
                  }
                  this._resize(newCapacity);
              }
          }
      }, {
          key: '_resize',
          value: function _resize(capacity) {
              if (capacity < 0) {
                  throw new RangeError('The requested capacity must not be negative');
              }
              var byteLength = this._constructor.BYTES_PER_ELEMENT * capacity;
              var buffer = this._array === undefined ? this._allocator(byteLength) : this._reallocator(this._array.buffer, byteLength, this._allocator, this._deallocator);
              if (!(buffer instanceof ArrayBuffer)) {
                  throw new TypeError('The reallocator must return an ArrayBuffer!');
              }
              if (buffer.byteLength < byteLength) {
                  throw new RangeError('The allocated buffer is too small!');
              }
              this._array = new this._constructor(buffer, 0, capacity);
              if (capacity < this._length) {
                  this._length = capacity;
              }
          }
      }]);
      return DynamicTypedArray;
  }();

export default DynamicTypedArray;
//# sourceMappingURL=dynamic-typed-array.es.js.map
